#############################################################
# make data for my CAS R talk

library(tweedie)

set.seed(0)
SimAll <- function(n) {
    # Return data frame with n records
    df <- data.frame(exposure=rep(1.0, n))
    df <- AddASM(df)
    df <- AddNumDrv(df)
    df <- AddCredit(df)
    df <- AddViol(df)

    df$rel <- with(df, rel_asm * rel_num_dv * rel_credit * rel_viol)
    df$loss <- rtweedie(n,
                        xi=1.5, # tweedie p value
                        mu=df$rel * 100,
                        # for fitting speed, use unrealistically low
                        # dispersion.  (realistic = 1000 maybe)
                        phi=400)
    return(df)
}

AddASM <- function(df) {
    # Add sex column
    df$sex <- rbinom(nrow(df), 1, 0.5)
    rel_sex <- 1 + df$sex * .1

    df$marital <- rbinom(nrow(df), 1, 0.5)
    df$age <- round(runif(nrow(df), 15, 100))
    unmar_rel <- ifelse(df$age <= 70,
                        1 + ((df$age - 50) / 20)^2,
                        2 + 0.02 * (df$age - 70))
    mar_rel <- ifelse(df$age <= 70,
                      1 + ((df$age - 55) / 30)^2,
                      1.25 * 1.02^(df$age - 70))
    df$rel_asm <- rel_sex * ifelse(df$marital == 0, unmar_rel, mar_rel)
    return(df)
}

AddNumDrv <- function(df) {
    # Add number vehices / number of drivers spline
    df$num_drv <- 1 + rbinom(nrow(df), 1, 0.5)
    df$num_veh <- 1 + rbinom(nrow(df), 1, 0.5)
    df <- transform(df,
                    rel_num_dv=ifelse(num_drv == 1,
                                      ifelse(num_veh == 1, 1.0, 0.9),
                                      ifelse(num_veh == 1, 1.5, 1.25)))
    return(df)
}

AddCredit <- function(df) {
    # Add credit spline
    df$credit <- round(runif(nrow(df), 200, 800))
    df$rel_credit <- 1 + ((df$credit - 800) / 100)^2 / 10
    return(df)
 }

AddViol <- function(df) {
    # Add major/minor violations
    df$major <- rpois(nrow(df), 0.5)
    df$minor <- rpois(nrow(df), 0.8)
    df$rel_viol <- with(df, 1.2^major * 1.12^minor)
    return(df)
}

sim.df <- SimAll(75e3)
input.colnames <- names(sim.df)[!(names(sim.df) %StartsWith% "rel")]
write.csv(sim.df[, input.colnames], file="input.data.csv",
          row.names=FALSE)
